<?php

namespace Drupal\mustache\Element;

use Drupal\Component\Utility\Crypt;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Render\Element\RenderElement;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\mustache\Exception\MustacheUrlNotFoundException;
use Drupal\mustache\Render\Markup;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Provides an element for rendering Mustache templates.
 *
 * @RenderElement("mustache")
 */
class Mustache extends RenderElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#theme' => 'mustache',
      '#pre_render' => [
        [$class, 'generateContentMarkup'],
      ],
    ];
  }

  /**
   * Generates content markup for a Mustache template.
   *
   * @param array $element
   *   The element to render.
   *
   * @return array
   *   The element to render.
   *
   * @throws \Exception
   *   In case something went wrong on placeholder rendering,
   *   or when the template or data url could not be found.
   */
  public static function generateContentMarkup(array $element) {
    if (!isset($element['#cache'])) {
      // Caching will be bubbled up if necessary.
      $element['#cache'] = [];
    }

    // Load and encode the template content.
    /** @var \Drupal\mustache\MustacheTemplates $templates */
    $templates = \Drupal::service('mustache.templates');
    /** @var \Drupal\mustache\Summable\SummableScriptsInterface $summables */
    $summables = \Drupal::service('mustache.summables');
    $template_name = $element['#template'];
    $template_content = $templates->getContent($template_name);
    $template_encoded = trim(substr(json_encode($template_content), 1, -1));
    $element['#template'] = [
      'name' => $template_name,
      'content' => $template_content,
      'encoded' => $template_encoded,
      'summable' => $summables->isEnabled(),
    ];

    $data = isset($element['#data']) ? $element['#data'] : NULL;
    $select = isset($element['#select']) ? $element['#select'] : NULL;
    if (is_string($select)) {
      $select = [$select];
    }

    // Obtain the Json data url, if given.
    $url = NULL;
    if ($data instanceof Url) {
      // The #data value represents a url object to fetch data from.
      $url = $data;
      $data = NULL;
    }
    elseif (is_string($data)) {
      // The #data value represents a url string to fetch data from.
      try {
        $url = Url::fromUri($data);
      }
      catch (\InvalidArgumentException $e) {
        $url = Url::fromUserInput($data);
      }
      $data = NULL;
    }

    if (!empty($element['#sync'])) {
      $sync_options = &$element['#sync'];

      // Get and check for the Json data provider url.
      if (!empty($sync_options['data_url'])) {
        $sync_url = $sync_options['data_url'];
        try {
          $sync_url = ($sync_url instanceof Url) ? $sync_url : Url::fromUri($sync_url);
        }
        catch (\InvalidArgumentException $e) {
          $sync_url = Url::fromUserInput($sync_url);
        }
      }
      elseif (isset($url)) {
        $sync_url = clone $url;
      }
      if (!isset($sync_url)) {
        throw new MustacheUrlNotFoundException(t('Could not find Json data source url for rendering template @template via Mustache.js.', ['@template' => $template_name]));
      }
      $sync_url->setAbsolute($sync_url->isExternal());
      $element['#use_sync'] = TRUE;
      $sync_options['data_url'] = $sync_url;

      // Include nested selection, if specified.
      $sync_select = $select;
      if (!empty($sync_options['select'])) {
        $sync_select = $sync_options['select'];
        if (is_string($sync_select)) {
          $sync_select = [$sync_select];
        }
      }
      $sync_options['select'] = json_encode($sync_select);

      // Use a period fallback value, if necessary.
      if (empty($sync_options['period'])) {
        $sync_options['period'] = 0;
      }

      // If not specified otherwise, then do not delay.
      if (empty($sync_options['delay'])) {
        $sync_options['delay'] = 0;
      }

      // Include and encode triggering elements, if specified.
      if (isset($sync_options['trigger'])) {
        foreach ($sync_options['trigger'] as &$trigger) {
          $trigger[1] = !empty($trigger[1]) && is_string($trigger[1]) ? $trigger[1] : 'load';
          $trigger[2] = !empty($trigger[2]) && is_int($trigger[2]) ? $trigger[2] : 1;
          if ($trigger[2] < 0) {
            // Always use -1 for triggering without any limits.
            $trigger[2] = -1;
          }
        }
      }
      if (empty($sync_options['trigger'])) {
        $sync_options['trigger'] = 'null';
      }
      else {
        $sync_options['trigger'] = json_encode($sync_options['trigger']);
      }

      // Build and attach the structure for DOM content synchronization.
      $element_id = 'msync-' . Crypt::randomBytesBase64(4);
      $sync_options['attributes'] = new Attribute(['id' => $element_id, 'class' => ['mustache-sync', 'not-synced']]);
      $sync_options['wrapper_tag'] = !empty($sync_options['wrapper_tag']) ? $sync_options['wrapper_tag'] : 'div';
      $element['#attached']['library'][] = $summables->isEnabled() ? $summables->getLibraryName($template_name) : 'mustache/sync';
    }
    else {
      $element['#use_sync'] = FALSE;
      $element['#sync'] = [];
    }

    if (!empty($element['#placeholder'])) {
      // Instead of rendering the content via Mustache.php,
      // render an arbitrary placeholder, by respecting its cache metadata.
      $element_metadata = CacheableMetadata::createFromRenderArray($element);
      $placeholder_metadata = CacheableMetadata::createFromRenderArray($element['#placeholder']);
      $element_metadata->merge($placeholder_metadata)->applyTo($element);

      /** @var \Drupal\Core\Render\Renderer $renderer */
      $renderer = \Drupal::service('renderer');
      $rendered = $renderer->render($element['#placeholder']);
      $element['#content'] = Markup::create($rendered);
    }
    else {
      // Render the content via Mustache.php.
      if (!isset($data) && isset($url)) {
        // Rendering via Mustache.php requires some data,
        // which now must be fetched from the url.
        $client = \Drupal::httpClient();
        $uri = $url->setAbsolute(TRUE)->toString();
        try {
          $response = $client->request('GET', $uri);
        }
        catch (GuzzleException $e) {
          \Drupal::logger('mustache')->error(t('Failed to fetch data from @url. Exception message: @message', ['@url' => $uri, '@message' => $e->getMessage()]));
          return static::emptyElement();
        }

        if (!($response->getStatusCode() == 200)) {
          \Drupal::logger('mustache')->error(t('Failed to fetch data from @url. Returned status code was: @status', ['@url' => $uri, '@status' => $response->getStatusCode()]));
          return static::emptyElement();
        }

        $body = $response->getBody()->getContents();
        $data = json_decode($body, TRUE);
        if (NULL === $data) {
          \Drupal::logger('mustache')->error(t('Invalid Json received from from @url.', ['@url' => $uri]));
          return static::emptyElement();
        }
      }
      if (!isset($data)) {
        // In case we still don't have any data,
        // we cannot continue rendering.
        \Drupal::logger('mustache')->error(t('Failed to get data for rendering Mustache template @template', ['@template' => $template_name]));
        return static::emptyElement();
      }

      if (!empty($select) && !empty($data)) {
        if (!is_array($data)) {
          // In case something different was passed,
          // try to convert it. When it fails, it should fail hard.
          $data = (array) $data;
        }
        $subset_exists = NULL;
        $data = NestedArray::getValue($data, $select, $subset_exists);
        if (!$subset_exists) {
          // Subset must be an array.
          $data = [];
        }
      }

      /** @var \Mustache_Engine $mustache */
      $mustache = \Drupal::service('mustache.engine');
      $output = $mustache->loadTemplate($template_content)->render($data);
      $element['#content'] = Markup::create($output);
    }

    return $element;
  }

  /**
   * Returns an empty element which is not cacheable.
   *
   * This element would only be used at unusual circumstances.
   * Because it's not what the result should look like,
   * the caching is completely disabled for being able
   * to generate an expected result as soon as possible.
   *
   * @return array
   *   The empty element.
   */
  public static function emptyElement() {
    return ['#cache' => ['max-age' => 0], '#printed' => TRUE, '#markup' => Markup::create('')];
  }

}
