<?php

namespace Drupal\mustache\Helpers;

use Drupal\Core\Url;
use Drupal\mustache\Exception\MustacheException;

/**
 * A helper class for building arrays to render Mustache templates.
 */
class MustacheRenderTemplate {

  /**
   * The render array representation.
   *
   * @var array
   */
  protected $render = ['#type' => 'mustache'];

  /**
   * The synchronization options.
   *
   * @var \Drupal\mustache\Helpers\SynchronizationOptions
   */
  protected $syncOptions;

  /**
   * MustacheRenderTemplate constructor.
   *
   * @param string $template
   *   The name of the Mustache template. It must be either defined
   *   by a hook implementation of hook_mustache_templates(), or
   *   is solely existent as a template file inside the theme.
   *   Example: The template name "foo_bar" is a template file being
   *   located inside the theme directory as "foo-bar.mustache.tpl".
   */
  public function __construct($template) {
    $this->render['#template'] = $template;
    $this->syncOptions = new SynchronizationOptions($this->render);
  }

  /**
   * Creates a new MustacheRenderTemplate instance.
   *
   * @param string $template
   *   The name of the Mustache template. It must be either defined
   *   by a hook implementation of hook_mustache_templates(), or
   *   is solely existent as a template file inside the theme.
   *   Example: The template name "foo_bar" is a template file being
   *   located inside the theme directory as "foo-bar.mustache.tpl".
   *
   * @return static
   */
  public static function build($template) {
    return new static($template);
  }

  /**
   * Set the data used for rendering only with Mustache.php.
   *
   * @param mixed $data
   *   An array or object which holds the data for PHP rendering.
   * @param array|null $select
   *  (Optional) The array holding the keys for nested data selection.
   *
   * @return $this
   *   The instance itself.
   */
  public function usingDataForPHPRendering($data, array $select = NULL) {
    $this->render['#data'] = $data;
    if (isset($select)) {
      $this->selectingSubsetFromData($select);
    }
    return $this;
  }

  /**
   * Define the url from where to receive Json-encoded data.
   *
   * @param \Drupal\Core\Url|string $url
   *   The url as object or string.
   * @param array|null $select
   *  (Optional) The array holding the keys for nested data selection.
   *
   * @return $this
   *   The instance itself.
   */
  public function usingJsonDataProviderUrl($url, array $select = NULL) {
    if (!($url instanceof Url) && !is_string($url)) {
      throw new MustacheException(t('The $url param must be a Url object or string.'));
    }
    if (is_string($url)) {
      try {
        $url = Url::fromUri($url);
      }
      catch (\InvalidArgumentException $e) {
        $url = Url::fromUserInput($url);
      }
    }
    $this->render['#data'] = $url;
    if (isset($select)) {
      $this->selectingSubsetFromData($select);
    }
    return $this;
  }

  /**
   * Specifies the subset to select out of the data.
   *
   * @param array $select
   *   The array holding the keys for nested data selection.
   *
   * @return $this
   *   The instance itself.
   */
  public function selectingSubsetFromData(array $select) {
    $this->render['#select'] = $select;
    return $this;
  }

  /**
   * Define a render array to use as a placeholder.
   *
   * @param array $placeholder
   *   An arbitrary render array of the placeholder.
   *
   * @return $this
   *   The instance itself.
   */
  public function withPlaceholder(array $placeholder) {
    $this->render['#placeholder'] = $placeholder;
    return $this;
  }

  /**
   * Determine and configure client-side DOM content synchronization.
   *
   * @return \Drupal\mustache\Helpers\SynchronizationOptions
   *   The synchronization options to define.
   */
  public function withClientSynchronization() {
    if (empty($this->render['#sync'])) {
      $this->render['#sync'] = [
        'period' => 0,
        'delay' => 0,
      ];
    }
    return $this->syncOptions;
  }

  /**
   * Get the render array result.
   *
   * @return array
   *   The render array result.
   */
  public function toRenderArray() {
    return $this->render;
  }

}

/**
 * Class SynchronizationOptions
 *
 * @internal
 */
final class SynchronizationOptions {

  /**
   * The render array representation of the parent instance.
   *
   * @var array
   */
  private $render;

  /**
   * SynchronizationOptions constructor.
   *
   * @param array &$render_array
   *   The corresponding render array representation.
   */
  public function __construct(array &$render_array) {
    $this->render = &$render_array;
  }

  /**
   * Define the url from where to receive Json-encoded data.
   *
   * @param \Drupal\Core\Url|string $url
   *   The url as object or string.
   * @param array|null $select
   *  (Optional) The array holding the keys for nested data selection.
   *
   * @return $this
   *   The instance itself.
   */
  public function usingJsonDataProviderUrl($url, array $select = NULL) {
    if (!($url instanceof Url) && !is_string($url)) {
      throw new MustacheException(t('The $url param must be a Url object or string.'));
    }
    if (is_string($url)) {
      try {
        $url = Url::fromUri($url);
      }
      catch (\InvalidArgumentException $e) {
        $url = Url::fromUserInput($url);
      }
    }
    $this->render['#sync']['data_url'] = $url;
    if (isset($select)) {
      $this->selectingSubsetFromData($select);
    }
    return $this;
  }

  /**
   * Specifies the subset to select out of the data.
   *
   * @param array $select
   *   The array holding the keys for nested data selection.
   *
   * @return $this
   *   The instance itself.
   */
  public function selectingSubsetFromData(array $select) {
    $this->render['#sync']['select'] = $select;
    return $this;
  }

  /**
   * Determine the interval in milliseconds to delay synchronization.
   *
   * @param int $milliseconds
   *   The interval in milliseconds to delay the synchronization start.
   *
   * @return $this
   *   The instance itself.
   */
  public function startsDelayed($milliseconds) {
    $this->render['#sync']['delay'] = $milliseconds;
    return $this;
  }

  /**
   * Determine the interval in milliseconds to repeat synchronization.
   *
   * @param int $milliseconds
   *   The interval in milliseconds to repeat synchronization.
   *
   * @return $this
   *   The instance itself.
   */
  public function periodicallyRefreshesAt($milliseconds) {
    $this->render['#sync']['period'] = $milliseconds;
    return $this;
  }

  /**
   * Determine and configure a triggering element (optional).
   *
   * @param string $css_selector
   *   The CSS element selector.
   *
   * @return \Drupal\mustache\Helpers\TriggeringElementOptions
   *   The configurable options for the triggering element.
   */
  public function startsWhenElementWasTriggered($css_selector) {
    return new TriggeringElementOptions($css_selector,$this->render);
  }

  /**
   * Determine the HTML tag to use (optional).
   *
   * By default, a div tag would wrap the rendered template.
   *
   * @param string $html_tag
   *   The HTML tag, e.g. span.
   *
   * @return $this
   *   The instance itself.
   */
  public function withWrapperTag($html_tag) {
    $this->render['#sync']['wrapper_tag'] = $html_tag;
    return $this;
  }

  /**
   * Get the render array result.
   *
   * @return array
   *   The render array result.
   */
  public function toRenderArray() {
    return $this->render;
  }

}

/**
 * Class TriggeringElementOptions
 *
 * @internal
 */
final class TriggeringElementOptions {

  /**
   * The CSS element selector.
   *
   * @var string
   */
  protected $selector;

  /**
   * The render array representation of the parent instance.
   *
   * @var array
   */
  protected $render;

  /**
   * The part of the render array which holds the trigger options.
   *
   * @var array
   */
  protected $trigger;

  /**
   * TriggeringElementOptions constructor.
   *
   * @param string $css_selector
   *   The CSS element selector.
   * @param array &$render_array
   *   The corresponding render array representation.
   */
  public function __construct($css_selector, array &$render_array) {
    $this->render = &$render_array;
    $this->selector = $css_selector;
    if (!isset($this->render['#sync']['trigger'])) {
      $this->render['#sync']['trigger'] = [];
    }
    $i = count($this->render['#sync']['trigger']);
    $this->render['#sync']['trigger'][$i] = [$css_selector, 'load', 1];
    $this->trigger = &$this->render['#sync']['trigger'][$i];
  }

  /**
   * Determine the Javascript event which would trigger at the element.
   *
   * @param string $event
   *   The Javascript triggering event, e.g. 'click'.
   *
   * @return $this
   *   The instance itself.
   */
  public function atEvent($event) {
    $this->trigger[1] = $event;
    return $this;
  }

  /**
   * Determine how many times the synchronization is being triggered.
   *
   * @param int $n
   *   The maximum number of times to trigger the synchronization.
   *
   * @return $this
   *   The instance itself.
   */
  public function upToNTimes($n) {
    $this->trigger[2] = $n;
    return $this;
  }

  /**
   * Determine that the synchronization is being triggered only once.
   *
   * @return $this
   *   The instance itself.
   */
  public function once() {
    $this->upToNTimes(1);
    return $this;
  }

  /**
   * Determine that the synchronization is always being triggered.
   *
   * @return $this
   *   The instance itself.
   */
  public function always() {
    $this->upToNTimes(-1);
    return $this;
  }

  /**
   * Get the render array result.
   *
   * @return array
   *   The render array result.
   */
  public function toRenderArray() {
    return $this->render;
  }

}
